@@ -3,17 +3,17 @@
 
 #include <yara/pe.h>
 
-#define MAX_PE_SECTIONS              96
+#define MAX_PE_SECTIONS 96
 
 
-#define IS_64BITS_PE(pe) \
-    (yr_le16toh(pe->header64->OptionalHeader.Magic) == IMAGE_NT_OPTIONAL_HDR64_MAGIC)
+#define IS_64BITS_PE(pe)                             \
+  (yr_le16toh(pe->header64->OptionalHeader.Magic) == \
+   IMAGE_NT_OPTIONAL_HDR64_MAGIC)
 
 
-#define OptionalHeader(pe,field)                \
-  (IS_64BITS_PE(pe) ?                           \
-   pe->header64->OptionalHeader.field :         \
-   pe->header->OptionalHeader.field)
+#define OptionalHeader(pe, field)                        \
+  (IS_64BITS_PE(pe) ? pe->header64->OptionalHeader.field \
+                    : pe->header->OptionalHeader.field)
 
 
 //
@@ -25,10 +25,10 @@
 
 typedef struct _IMPORTED_DLL
 {
-  char *name;
+  char* name;
 
-  struct _IMPORT_FUNCTION *functions;
-  struct _IMPORTED_DLL *next;
+  struct _IMPORT_FUNCTION* functions;
+  struct _IMPORTED_DLL* next;
 
 } IMPORTED_DLL, *PIMPORTED_DLL;
 
@@ -43,11 +43,11 @@ typedef struct _IMPORTED_DLL
 
 typedef struct _IMPORT_FUNCTION
 {
-  char *name;
+  char* name;
   uint8_t has_ordinal;
   uint16_t ordinal;
 
-  struct _IMPORT_FUNCTION *next;
+  struct _IMPORT_FUNCTION* next;
 
 } IMPORT_FUNCTION, *PIMPORT_FUNCTION;
 
@@ -57,7 +57,8 @@ typedef struct _PE
   const uint8_t* data;
   size_t data_size;
 
-  union {
+  union
+  {
     PIMAGE_NT_HEADERS32 header;
     PIMAGE_NT_HEADERS64 header64;
   };
@@ -71,38 +72,29 @@ typedef struct _PE
 } PE;
 
 
-#define fits_in_pe(pe, pointer, size) \
-    ((size_t) size <= pe->data_size && \
-     (uint8_t*) (pointer) >= pe->data && \
-     (uint8_t*) (pointer) <= pe->data + pe->data_size - size)
+#define fits_in_pe(pe, pointer, size)                                    \
+  ((size_t) size <= pe->data_size && (uint8_t*) (pointer) >= pe->data && \
+   (uint8_t*) (pointer) <= pe->data + pe->data_size - size)
 
 #define struct_fits_in_pe(pe, pointer, struct_type) \
-    fits_in_pe(pe, pointer, sizeof(struct_type))
+  fits_in_pe(pe, pointer, sizeof(struct_type))
 
 
-PIMAGE_NT_HEADERS32 pe_get_header(
-    const uint8_t* data,
-    size_t data_size);
+PIMAGE_NT_HEADERS32 pe_get_header(const uint8_t* data, size_t data_size);
 
 
-PIMAGE_DATA_DIRECTORY pe_get_directory_entry(
-    PE* pe,
-    int entry);
+PIMAGE_DATA_DIRECTORY pe_get_directory_entry(PE* pe, int entry);
 
 
-int64_t pe_rva_to_offset(
-    PE* pe,
-    uint64_t rva);
+int64_t pe_rva_to_offset(PE* pe, uint64_t rva);
 
 
-char *ord_lookup(
-    char *dll,
-    uint16_t ord);
+char* ord_lookup(char* dll, uint16_t ord);
 
 
 #if HAVE_LIBCRYPTO
 #include <openssl/asn1.h>
-time_t ASN1_get_time_t(ASN1_TIME* time);
+time_t ASN1_get_time_t(const ASN1_TIME* time);
 #endif
 
 #endif
